#include <stdio.h>
#include "JDNAObj.h"
#include "JInteger.h"
#include "JIntegerData.h"
#include "JStringData.h"
#include "JIntegerProperty.h"
#include "JIntegerListProperty.h"
#include "JStringProperty.h"
#include "JLinkObj.h"
#include "JBytesData.h"


char* theJDNAObj = JDNAObj().Register();


const char* JDNAObj::className() const { return "JDNAObj";}


JObject* JDNAObj::clone() const { return new JDNAObj(*this);}


// class constructor
JDNAObj::JDNAObj() {
  igm = 1 + 1 + 1;
  ogm = 1 + 1;
  enabled = 0;
}


void JDNAObj::draw(JGraphics g, int x, int y, int w, int h) {
  JRect rect(x, y, w, h);
  drawText(g, JString("DNA"), rect);
}


boolean JDNAObj::inputAllowed(int n)
{ return !inputSet(n).last();}
JString JDNAObj::inputTag(int n) {
  char* tag[] = {
    "filename",
    "enabled",
  "tick" };
  return( tag[n] );
}


int JDNAObj::inputType( int n ) {
  if ( n == 0 ) return JStringData::id;
  return JIntegerData::id;
}
JString JDNAObj::outputTag(int n) {
  char* tag[] = {
    "AT",
    "CG" };
  return( tag[n] );
}


JFRect JDNAObj::inputArea(int n) {
  if ( n < ( 1 )) {
    return leftArea( n, 0, 1, 0.1, 0.8 );
  } else {
    return topArea( n, 1, 2, 0.25 );
  }
}


JFRect JDNAObj::outputArea(int n) {
  return rightArea(n, 0, 1 + 1);
}


int JDNAObj::inputFace(int n) {
  if ( n < 1 ) {
    return LEFT;
  } else {
    return TOP;
  }
}


int JDNAObj::outputFace(int n) {
  return RIGHT;
}


JFPoint JDNAObj::inputPoint(int n, JLinkObj& link) {
  if (n<1) return leftPoint(n, link, 0, 1, 0.1, 0.8);
  return topPoint(n, link, 1, 2);
}


JFPoint JDNAObj::outputPoint(int n, JLinkObj& link) {
  return rightPoint(n, link, 0, 1 + 1);
}


void JDNAObj::access(int n, JLinkObj& link, const JDataType& data) {
  if ( n == 0 ) {
    INT(data) = AT;
  } else if ( n == 1 ) {
    INT(data) = CG;
  }
}

bool JDNAObj::readLine( FILE* fp ) {
	if ( fgets( &line[0], 500, fp ) == NULL ) {
		return( false );
	} else {
		return( true );
	}
}

void JDNAObj::processLine() {
	char* scan = &line[0];
	while ( *scan ) {
		if ( *scan == 'A' ) {
			AT++;
			broadcast( 0 );
		} else if ( *scan == 'T' ) {
			AT--;
			broadcast( 0 );
		} else if ( *scan == 'C' ) {
			CG++;
			broadcast( 1 );
		} else if ( *scan == 'G' ) {
			CG--;
			broadcast( 1 );
		}
		scan++;
	}
}

void JDNAObj::engine(int n, JLinkObj& link) {
  switch (n) {
  case FILENAME:
    link.access( JStringData( filename ));
    break;
  case ENABLED:
    link.access(JIntegerData(enabled));
	if ( enabled && filename.length() > 0 ) {
		fp = fopen( filename, "r" );
	}
	break;
  case TICK:
	if ( enabled && fp != NULL ) {
		if ( readLine( fp )) {
			processLine();
		} else {
			fclose( fp );
			fp = NULL;
		}
	}
    break;
  default:
    break;
  }
}


