#include <stdio.h>
#include "JDNA_PMObj.h"
#include "JInteger.h"
#include "JIntegerData.h"
#include "JStringData.h"
#include "JIntegerProperty.h"
#include "JIntegerListProperty.h"
#include "JStringProperty.h"
#include "JLinkObj.h"
#include "JBytesData.h"


char* theJDNA_PMObj = JDNA_PMObj().Register();


const char* JDNA_PMObj::className() const { return "JDNA_PMObj";}


JObject* JDNA_PMObj::clone() const { return new JDNA_PMObj(*this);}


// class constructor
JDNA_PMObj::JDNA_PMObj() {
  igm = 1 + 1 + 1;
  ogm = 1 + 1 + 1;
  enabled = 0;
  contig = 0;
  file_is_open = 0;
  fp = NULL;
  lastLine = NULL;
}


void JDNA_PMObj::draw(JGraphics g, int x, int y, int w, int h) {
  JRect rect(x, y, w, h);
  drawText(g, JString("DNA_PM"), rect);
}


boolean JDNA_PMObj::inputAllowed(int n)
{ return !inputSet(n).last();}
JString JDNA_PMObj::inputTag(int n) {
  char* tag[] = {
    "filename",
    "enabled",
  "tick" };
  return( tag[n] );
}


int JDNA_PMObj::outputType( int n ) {
  if ( n == 1 ) return JStringData::id;
  return JIntegerData::id;
}
int JDNA_PMObj::inputType( int n ) {
  if ( n == 0 ) return JStringData::id;
  return JIntegerData::id;
}
JString JDNA_PMObj::outputTag(int n) {
  char* tag[] = {
    "contig",
    "line",
    "file_is_open" };
  return( tag[n] );
}


JFRect JDNA_PMObj::inputArea(int n) {
  if ( n < ( 1 )) {
    return leftArea( n, 0, 1, 0.1, 0.8 );
  } else {
    return topArea( n, 1, 2, 0.25 );
  }
}


JFRect JDNA_PMObj::outputArea(int n) {
  if ( n < 2 ) {
    return rightArea(n, 0, 2, 0.1, 0.8);
  } else {
    return bottomArea(n, 2, 1, 0.1, 0.8);
  }
}


int JDNA_PMObj::inputFace(int n) {
  if ( n < 1 ) {
    return LEFT;
  } else {
    return TOP;
  }
}


int JDNA_PMObj::outputFace(int n) {
  if ( n < 2 ) {
    return RIGHT;
  } else {
    return BOTTOM;
  }
}


JFPoint JDNA_PMObj::inputPoint(int n, JLinkObj& link) {
  if (n<1) return leftPoint(n, link, 0, 1, 0.1, 0.8);
  return topPoint(n, link, 1, 2);
}


JFPoint JDNA_PMObj::outputPoint(int n, JLinkObj& link) {
  if ( n < 2 ) {
    return rightPoint(n, link, 0, 2, 0.1, 0.8);
  } else {
    return bottomPoint(n, link, 2, 1, 0.1, 0.8);
  }
}


void JDNA_PMObj::access(int n, JLinkObj& link, const JDataType& data) {
  if ( n == 0 ) {
    INT(data) = contig;
  } else if ( n == 1 ) {
    JSTRING(data) = outline;
  } else if ( n == 2 ) {
    INT(data) = file_is_open;
  }
}


bool JDNA_PMObj::readLine( FILE* fp ) {
	if ( fgets( &line[0], 500, fp ) == NULL ) {
		return( false );
	} else {
		return( true );
	}
}

#include <malloc.h>
#include <string.h>

char* newstrcpy( char* line ) {
	char* result = (char*)malloc( strlen( line ) + 1 );
	strcpy( result, line );
	return( result );
}
void JDNA_PMObj::processLine() {
	//if ( lastLine != NULL ) {
	//	free( lastLine );
	//}
	//lastLine = newstrcpy( &line[0] );
	//if ( lastLine[0] == '>' ) {
	if ( line[0] == '>' ) {
		contig = true;
		broadcast( 0 );
	} else {
		if ( contig ) {
			contig = false;
			broadcast( 0 );
		}
		//outline = JString( lastLine );
		outline = JString( &line[0], 2 );
		broadcast( 1 );
	}
}



void JDNA_PMObj::engine(int n, JLinkObj& link) {
  switch (n) {
  case FILENAME:
    link.access( JStringData( filename ));
    break;
  case ENABLED:
    link.access(JIntegerData(enabled));
	if ( enabled && filename.length() > 0 ) {
		fp = fopen( filename, "r" );
	} else {
		if ( fp != NULL ) {
			fclose( fp );
		}
		fp = NULL;
	}
	if ( fp == NULL ) {
		file_is_open = false;
	} else {
		file_is_open = true;
	}
	broadcast( 2 );
    break;
  case TICK:
	  link.access( JIntegerData( tick ));
	  if ( tick ) {
	  	if ( enabled && fp != NULL ) {
			if ( readLine( fp )) {
				processLine();
			} else {
				fclose( fp );
				fp = NULL;
				file_is_open = false;
				broadcast( 2 );
			}
		}
	  }
	  break;

  default:
    break;
  }
}


