#include "JStrAreaObj.h"
#include "JInteger.h"
#include "JIntegerData.h"
#include "JStringData.h"
#include "JIntegerProperty.h"
#include "JIntegerListProperty.h"
#include "JStringProperty.h"
#include "JLinkObj.h"
#include "JBytesData.h"


char* theJStrAreaObj = JStrAreaObj().Register();


void JStrAreaObj::writeContent(JOutputStream& os) {
  JAddObj::writeContent(os);
  putObject(os, "startStr", startStr);
  putObject(os, "endStr", endStr);
}

void JStrAreaObj::readContent(JDictionary& dict) {
  JAddObj::readContent(dict);
  JObject* obj_startStr = getObject( dict, "startStr");
  if ( obj_startStr ) startStr = *(JString*)obj_startStr;
  JObject* obj_endStr = getObject( dict, "endStr");
  if ( obj_endStr ) endStr = *(JString*)obj_endStr;
}

const char* JStrAreaObj::className() const { return "JStrAreaObj";}


JObject* JStrAreaObj::clone() const { return new JStrAreaObj(*this);}


// class constructor
JStrAreaObj::JStrAreaObj() {
  igm = 1 + 1 + 1;
  ogm = 1;
  enabled = 0;
  reset = 0;
  inArea = 0;
}


void JStrAreaObj::draw(JGraphics g, int x, int y, int w, int h) {
	if (( startStr.length() > 0 ) && ( endStr.length() > 0 )) {
	int dh = h/3;
	int dy = h/12;
	JRect rect1(x, y+dy, w, dh);
	JRect rect2(x, y+h/2, w, dh);
	drawText(g, startStr, rect1);
	drawText(g, endStr, rect2);
	} else {
  JRect rect(x, y, w, h);
  drawText(g, JString("StrArea"), rect);
	}
}


boolean JStrAreaObj::inputAllowed(int n)
{ return !inputSet(n).last();}
JString JStrAreaObj::inputTag(int n) {
  char* tag[] = {
    "text",
    "enabled",
    "reset" };
  return( tag[n] );
}


int JStrAreaObj::inputType( int n ) {
  if ( n == 0 ) return JStringData::id;
  return JIntegerData::id;
}
JString JStrAreaObj::outputTag(int n) {
  char* tag[] = {
    "inArea" };
  return( tag[n] );
}


JFRect JStrAreaObj::inputArea(int n) {
  if ( n < ( 1 )) {
    return leftArea( n, 0, 1, 0.1, 0.8 );
  } else {
    return topArea( n, 1, 1 + 1, 0.25 );
  }
}


JFRect JStrAreaObj::outputArea(int n) {
  return rightArea(n, 0, 1);
}


int JStrAreaObj::inputFace(int n) {
  if ( n < 1 ) {
    return LEFT;
  } else {
    return TOP;
  }
}


int JStrAreaObj::outputFace(int n) {
  return RIGHT;
}


JFPoint JStrAreaObj::inputPoint(int n, JLinkObj& link) {
  if (n<1) return leftPoint(n, link, 0, 1, 0.1, 0.8);
  return topPoint(n, link, 1, 1 + 1);
}


JFPoint JStrAreaObj::outputPoint(int n, JLinkObj& link) {
  return rightPoint(n, link, 0, 1);
}


void JStrAreaObj::access(int n, JLinkObj& link, const JDataType& data) {
  //if ( n == 0 ) {
    INT(data) = inArea;
  //}
}

#include <string.h>
void JStrAreaObj::processText( char* text ) {
	if ( strcmp( text, startStr ) == 0 ) {
		inArea = 65535;
		broadcast( 0 );
	} else if ( strcmp( text, endStr ) == 0 ) {
		inArea = 0;
		broadcast( 0 );
	}
}

void JStrAreaObj::engine(int n, JLinkObj& link) {
  switch (n) {
  case TEXT:
    link.access( JStringData( text ));
	if ( enabled ) {
		processText( text );
	}
    break;
  case ENABLED:
    link.access(JIntegerData(enabled));
    break;
  case RESET:
    link.access(JIntegerData(reset));
	if ( reset ) {
		inArea = false;
	}
    break;
  default:
    break;
  }
}


JArray JStrAreaObj::getProperties() {
  JArray properties;
  properties.append(JStringProperty("startStr", startStr));
  properties.append(JStringProperty("endStr", endStr));
  return properties;
}


boolean JStrAreaObj::updateProperty(JProperty& prop) {
  if (prop.getName() == JString("startStr")) {
    startStr = ((JStringProperty*)&prop)->value;
	repaint();
    return true;
  } else if (prop.getName() == JString("endStr")) {
    endStr = ((JStringProperty*)&prop)->value;
	repaint();
    return true;
  }
  return false;
}


